@@ -60,6 +60,7 @@
 (defvar ement-room-prism)
 (defvar ement-room-prism-color-adjustment)
 (defvar ement-room-prism-minimum-contrast)
+(defvar ement-room-unread-only-counts-notifications)
 
 ;;;; Compatibility
 
@@ -149,7 +150,8 @@ the request."
 If FORCE-P, leave without prompting.  ROOM may be an `ement-room'
 struct, or a room ID or alias string."
   ;; TODO: Rename `room' argument to `room-or-id'.
-  (interactive (ement-complete-room :session (ement-complete-session)))
+  (interactive (ement-complete-room :session (ement-complete-session)
+                 :prompt "Leave room: "))
   (cl-assert room) (cl-assert session)
   (cl-etypecase room
     (ement-room)
@@ -188,8 +190,8 @@ struct, or a room ID or alias string."
   "Forget ROOM on SESSION.
 If FORCE-P (interactively, with prefix), prompt to leave the room
 when necessary, and forget the room without prompting."
-  (interactive (pcase-let ((`(,room ,session) (ement-complete-room)))
-                 (list room session current-prefix-arg)))
+  (interactive (ement-complete-room :session (ement-complete-session)
+                 :prompt "Forget room: "))
   (pcase-let* (((cl-struct ement-room id display-name status) room)
                (endpoint (format "rooms/%s/forget" (url-hexify-string id))))
     (pcase status
@@ -298,61 +300,6 @@ members, show in a new buffer; otherwise show in echo area."
         :then list-members))
     (message "Listing members of %s..." (ement--format-room room))))
 
-(defun ement-describe-room (room session)
-  "Describe ROOM on SESSION."
-  (interactive (pcase-let ((`(,room ,session) (ement-complete-room :session ement-session)))
-                 (list room session)))
-  (cl-labels ((heading (string)
-                       (propertize (or string "") 'face 'font-lock-builtin-face))
-              (id (string)
-                  (propertize (or string "") 'face 'font-lock-constant-face)))
-    (pcase-let* (((cl-struct ement-room (id room-id) avatar display-name canonical-alias members timeline status topic
-                             (local (map fetched-members-p)))
-                  room)
-                 ((cl-struct ement-session user) session)
-                 ((cl-struct ement-user (id user-id)) user)
-                 (inhibit-read-only t))
-      (if (not fetched-members-p)
-          ;; Members not fetched: fetch them and re-call this command.
-          (ement--get-joined-members room session
-            :then (lambda (_) (ement-room-describe room session)))
-        (with-current-buffer (get-buffer-create (format "*Ement room description: %s*" (or display-name canonical-alias room-id)))
-          (erase-buffer)
-          (let ((members (cl-sort (cl-loop for user being the hash-values of members
-                                           collect (format "%s <%s>" (ement--format-user user room session)
-                                                           (id (ement-user-id user))))
-                                  (lambda (a b) (string-collate-lessp a b nil t)))))
-            (save-excursion
-              (insert "\"" (propertize (or display-name canonical-alias room-id) 'face 'font-lock-doc-face) "\"" " is a room "
-                      (propertize (pcase status
-                                    ('invite "invited")
-                                    ('join "joined")
-                                    ('leave "left")
-                                    (_ (symbol-name status)))
-                                  'face 'font-lock-comment-face)
-                      " on session <" (id user-id) ">.\n\n"
-                      (heading "Avatar: ") (or avatar "") "\n\n"
-                      (heading "ID: ") "<" (id room-id) ">" "\n"
-                      (heading "Alias: ") "<" (id canonical-alias) ">" "\n\n"
-                      (heading "Topic: ") (propertize (or topic "[none]") 'face 'font-lock-comment-face) "\n\n"
-                      (heading "Retrieved events: ") (number-to-string (length timeline)) "\n"
-                      (heading "  spanning: ")
-                      (format-time-string "%Y-%m-%d %H:%M:%S"
-                                          (/ (ement-event-origin-server-ts
-                                              (car (cl-sort (copy-sequence timeline) #'< :key #'ement-event-origin-server-ts)))
-                                             1000))
-                      (heading " to ")
-                      (format-time-string "%Y-%m-%d %H:%M:%S\n\n"
-                                          (/ (ement-event-origin-server-ts
-                                              (car (cl-sort (copy-sequence timeline) #'> :key #'ement-event-origin-server-ts)))
-                                             1000))
-                      (heading "Members") " (" (number-to-string (length members)) "):\n")
-              (dolist (member members)
-                (insert "  " member "\n"))))
-          (read-only-mode)
-          (pop-to-buffer (current-buffer)))))))
-(defalias 'ement-room-describe #'ement-describe-room)
-
 (defun ement-send-direct-message (session user-id message)
   "Send a direct MESSAGE to USER-ID on SESSION.
 Uses the latest existing direct room with the user, or creates a
@@ -415,14 +362,313 @@ If DELETE (interactively, with prefix), delete it."
                                  (url-hexify-string user-id) (url-hexify-string room-id) (url-hexify-string tag)))
                (method (if delete 'delete 'put)))
     ;; TODO: "order".
+    ;; FIXME: Removing a tag on a left room doesn't seem to work (e.g. to unfavorite a room after leaving it, but not forgetting it).
     (ement-api session endpoint :version "v3" :method method :data (unless delete "{}")
       :then (lambda (data)
               (ement-debug "Changed tag on room" method tag data room)))))
 
+(defun ement-set-display-name (display-name session)
+  "Set DISPLAY-NAME for user on SESSION.
+Sets global displayname."
+  (interactive
+   (let* ((session (ement-complete-session))
+          (display-name (read-string "Set display-name to: " nil nil
+                                     (ement-user-displayname (ement-session-user session)))))
+     (list display-name session)))
+  (pcase-let* (((cl-struct ement-session user) session)
+               ((cl-struct ement-user (id user-id)) user)
+               (endpoint (format "profile/%s/displayname" (url-hexify-string user-id))))
+    (ement-api session endpoint :method 'put :version "v3"
+      :data (json-encode (ement-alist "displayname" display-name))
+      :then (lambda (_data)
+              (message "Ement: Display name set to %S for <%s>" display-name
+                       (ement-user-id (ement-session-user session)))))))
+
+(defun ement-room-set-display-name (display-name room session)
+  "Set DISPLAY-NAME for user in ROOM on SESSION.
+Sets the name only in ROOM, not globally."
+  (interactive
+   (pcase-let* ((`(,room ,session) (or (when (bound-and-true-p ement-room)
+                                         (list ement-room ement-session))
+                                       (ement-complete-room)))
+                (prompt (format "Set display-name in %S to: "
+                                (ement--format-room room)))
+                (display-name (read-string prompt nil nil
+                                           (ement-user-displayname (ement-session-user session)))))
+     (list display-name room session)))
+  ;; NOTE: This does not seem to be documented in the spec, so we imitate the
+  ;; "/myroomnick" command in SlashCommands.tsx from matrix-react-sdk.
+  (pcase-let* (((cl-struct ement-room state) room)
+               ((cl-struct ement-session user) session)
+               ((cl-struct ement-user id) user)
+               (member-event (cl-find-if (lambda (event)
+                                           (and (equal id (ement-event-state-key event))
+                                                (equal "m.room.member" (ement-event-type event))
+                                                (equal "join" (alist-get 'membership (ement-event-content event)))))
+                                         state)))
+    (cl-assert member-event)
+    (setf (alist-get 'displayname (ement-event-content member-event)) display-name)
+    (ement-put-state room "m.room.member" id (ement-event-content member-event) session
+      :then (lambda (_data)
+              (message "Ement: Display name set to %S for <%s> in %S" display-name
+                       (ement-user-id (ement-session-user session))
+                       (ement--format-room room))))))
+
+;;;;;; Describe room
+
+(defvar ement-describe-room-mode-map
+  (let ((map (make-sparse-keymap)))
+    (define-key map (kbd "q") #'quit-window)
+    map)
+  "Keymap for `ement-describe-room-mode' buffers.")
+
+(define-derived-mode ement-describe-room-mode read-only-mode
+  "Ement-Describe-Room" "Major mode for `ement-describe-room' buffers.")
+
+(defun ement-describe-room (room session)
+  "Describe ROOM on SESSION."
+  (interactive (pcase-let ((`(,room ,session) (ement-complete-room :session ement-session)))
+                 (list room session)))
+  (cl-labels ((heading (string)
+                       (propertize (or string "") 'face 'font-lock-builtin-face))
+              (id (string)
+                  (propertize (or string "") 'face 'font-lock-constant-face))
+              (member<
+               (a b) (string-collate-lessp (car a) (car b) nil t)))
+    (pcase-let* (((cl-struct ement-room (id room-id) avatar display-name canonical-alias members timeline status topic
+                             (local (map fetched-members-p)))
+                  room)
+                 ((cl-struct ement-session user) session)
+                 ((cl-struct ement-user (id user-id)) user)
+                 (inhibit-read-only t))
+      (if (not fetched-members-p)
+          ;; Members not fetched: fetch them and re-call this command.
+          (ement--get-joined-members room session
+            :then (lambda (_) (ement-room-describe room session)))
+        (with-current-buffer (get-buffer-create (format "*Ement room description: %s*" (or display-name canonical-alias room-id)))
+          (let ((inhibit-read-only t))
+            (erase-buffer)
+            ;; We avoid looping twice by doing a bit more work here and
+            ;; returning a cons which we destructure.
+            (pcase-let* ((`(,member-pairs . ,name-width)
+                          (cl-loop for user being the hash-values of members
+                                   for formatted = (ement--format-user user room session)
+                                   for id = (format "<%s>" (id (ement-user-id user)))
+                                   collect (cons formatted id)
+                                   into pairs
+                                   maximizing (string-width id) into width
+                                   finally return (cons (cl-sort pairs #'member<) width)))
+                         ;; We put the MXID first, because users may use Unicode characters
+                         ;; in their displayname, which `string-width' does not always
+                         ;; return perfect results for, and putting it last prevents
+                         ;; alignment problems.
+                         (spec (format "%%-%ss %%s" name-width)))
+              (save-excursion
+                (insert "\"" (propertize (or display-name canonical-alias room-id) 'face 'font-lock-doc-face) "\"" " is a room "
+                        (propertize (pcase status
+                                      ('invite "invited")
+                                      ('join "joined")
+                                      ('leave "left")
+                                      (_ (symbol-name status)))
+                                    'face 'font-lock-comment-face)
+                        " on session <" (id user-id) ">.\n\n"
+                        (heading "Avatar: ") (or avatar "") "\n\n"
+                        (heading "ID: ") "<" (id room-id) ">" "\n"
+                        (heading "Alias: ") "<" (id canonical-alias) ">" "\n\n"
+                        (heading "Topic: ") (propertize (or topic "[none]") 'face 'font-lock-comment-face) "\n\n"
+                        (heading "Retrieved events: ") (number-to-string (length timeline)) "\n"
+                        (heading "  spanning: ")
+                        (format-time-string "%Y-%m-%d %H:%M:%S"
+                                            (/ (ement-event-origin-server-ts
+                                                (car (cl-sort (copy-sequence timeline) #'< :key #'ement-event-origin-server-ts)))
+                                               1000))
+                        (heading " to ")
+                        (format-time-string "%Y-%m-%d %H:%M:%S\n\n"
+                                            (/ (ement-event-origin-server-ts
+                                                (car (cl-sort (copy-sequence timeline) #'> :key #'ement-event-origin-server-ts)))
+                                               1000))
+                        (heading "Members") " (" (number-to-string (hash-table-count members)) "):\n")
+                (pcase-dolist (`(,formatted . ,id) member-pairs)
+                  (insert "  " (format spec id formatted) "\n")))))
+          (unless (eq major-mode 'ement-describe-room-mode)
+            ;; Without this check, activating the mode again causes a "Cyclic keymap
+            ;; inheritance" error.
+            (ement-describe-room-mode))
+          (pop-to-buffer (current-buffer)))))))
+
+(defalias 'ement-room-describe #'ement-describe-room)
+
+;;;;;; Push rules
+
+;; NOTE: Although v1.4 of the spec is available and describes setting the push rules using
+;; the "v3" API endpoint, the Element client continues to use the "r0" endpoint, which is
+;; slightly different.  This implementation will follow Element's initially, because the
+;; spec is not simple, and imitating Element's requests will make it easier.
+
+(defun ement-room-notification-state (room session)
+  "Return notification state for ROOM on SESSION.
+Returns one of nil (meaning default rules are used), `all-loud',
+`all', `mentions-and-keywords', or `none'."
+  ;; Following the implementation of getRoomNotifsState() in RoomNotifs.ts in matrix-react-sdk.
+
+  ;; TODO: Guest support (in which case the state should be `all').
+  ;; TODO: Store account data as a hash table of event types.
+  (let ((push-rules (cl-find-if (lambda (alist)
+                                  (equal "m.push_rules" (alist-get 'type alist)))
+                                (ement-session-account-data session))))
+    (cl-labels ((override-mute-rule-for-room-p
+                 ;; Following findOverrideMuteRule() in RoomNotifs.ts.
+                 (room) (when-let ((overrides (map-nested-elt push-rules '(content global override))))
+                          (cl-loop for rule in overrides
+                                   when (and (alist-get 'enabled rule)
+                                             (rule-for-room-p rule room))
+                                   return rule)))
+                (rule-for-room-p
+                 ;; Following isRuleForRoom() in RoomNotifs.ts.
+                 (rule room) (and (/= 1 (length (alist-get 'conditions rule)))
+                                  (pcase-let* ((condition (elt (alist-get 'conditions rule) 0))
+                                               ((map kind key pattern) condition))
+                                    (and (equal "event_match" kind)
+                                         (equal "room_id" key)
+                                         (equal (ement-room-id room) pattern)))))
+                (mute-rule-p
+                 (rule) (and (= 1 (length (alist-get 'actions rule)))
+                             (equal "dont_notify" (elt (alist-get 'actions rule) 0))))
+                (tweak-rule-p
+                 (type rule) (pcase-let (((map ('actions `[,action ,alist])) rule))
+                               (and (equal "notify" action)
+                                    (equal type (alist-get 'set_tweak alist))))))
+      ;; If none of these match, nil is returned, meaning that the default rule is used
+      ;; for the room.
+      (if (override-mute-rule-for-room-p room)
+          'none
+        (when-let ((room-rule (cl-find-if (lambda (rule)
+                                            (equal (ement-room-id room) (alist-get 'rule_id rule)))
+                                          (map-nested-elt push-rules '(content global room)))))
+          (cond ((not (alist-get 'enabled room-rule))
+                 ;; NOTE: According to comment in getRoomNotifsState(), this assumes that
+                 ;; the default is to notify for all messages, which "will be 'wrong' for
+                 ;; one to one rooms because they will notify loudly for all messages."
+                 'all)
+                ((mute-rule-p room-rule)
+                 ;; According to comment, a room-level mute still allows mentions to
+                 ;; notify.
+                 'mentions-and-keywords)
+                ((tweak-rule-p "sound" room-rule) 'all-loud)))))))
+
+(defun ement-room-set-notification-state (state room session)
+  "Set notification STATE for ROOM on SESSION.
+STATE may be nil to set the rules to default, `all',
+`mentions-and-keywords', or `none'."
+  ;; This merely attempts to reproduce the behavior of Element's simple notification
+  ;; options.  It does not attempt to offer all of the features defined in the spec.  And,
+  ;; yes, it is rather awkward, having to sometimes* make multiple requests of different
+  ;; "kinds" to set the rules for a single room, but that is how the API works.
+  ;;
+  ;; * It appears that Element only makes multiple requests of different kinds when
+  ;; strictly necessary, but coding that logic now would seem likely to be a waste of
+  ;; time, given that Element doesn't even use the latest version of the spec yet.  So
+  ;; we'll just do the "dumb" thing and always send requests of both "override" and
+  ;; "room" kinds, which appears to Just Work™.
+  ;;
+  ;; TODO: Match rules to these user-friendly notification states for presentation.  See
+  ;; <https://github.com/matrix-org/matrix-react-sdk/blob/8c67984f50f985aa481df24778078030efa39001/src/RoomNotifs.ts>.
+
+  ;; TODO: Support `all-loud' ("all_messages_loud").
+  (interactive
+   (pcase-let* ((`(,room ,session) (or (when (bound-and-true-p ement-room)
+                                         (list ement-room ement-session))
+                                       (ement-complete-room)))
+                (prompt (format "Set notification rules for %s: " (ement--format-room room)))
+                (available-states (ement-alist "Default" nil
+                                               "All messages" 'all
+                                               "Mentions and keywords" 'mentions-and-keywords
+                                               "None" 'none))
+                (selected-rule (completing-read prompt (mapcar #'car available-states) nil t))
+                (state (alist-get selected-rule available-states nil nil #'equal)))
+     (list state room session)))
+  (cl-labels ((set-rule (kind rule queue message-fn)
+                        (pcase-let* (((cl-struct ement-room (id room-id)) room)
+                                     (rule-id (url-hexify-string room-id))
+                                     (endpoint (format "pushrules/global/%s/%s" kind rule-id))
+                                     (method (if rule 'put 'delete))
+                                     (then (if rule
+                                               ;; Setting rules requires PUTting the rules, then making a second
+                                               ;; request to enable them.
+                                               (lambda (_data)
+                                                 (ement-api session (concat endpoint "/enabled") :queue queue :version "r0"
+                                                   :method 'put :data (json-encode (ement-alist 'enabled t))
+                                                   :then message-fn))
+                                             message-fn)))
+                          (ement-api session endpoint :queue queue :method method :version "r0"
+                            :data (json-encode rule)
+                            :then then
+                            :else (lambda (plz-error)
+                                    (pcase-let* (((cl-struct plz-error response) plz-error)
+                                                 ((cl-struct plz-response status) response))
+                                      (pcase status
+                                        (404 (pcase rule
+                                               (`nil
+                                                ;; Room already had no rules, so none being found is not an
+                                                ;; error.
+                                                nil)
+                                               (_ ;; Unexpected error: re-signal.
+                                                (ement-api-error plz-error))))
+                                        (_ ;; Unexpected error: re-signal.
+                                         (ement-api-error plz-error)))))))))
+    (pcase-let* ((available-states
+                  (ement-alist
+                   nil (ement-alist
+                        "override" nil
+                        "room" nil)
+                   'all (ement-alist
+                         "override" nil
+                         "room" (ement-alist
+                                 'actions (vector "notify" (ement-alist
+                                                            'set_tweak "sound"
+                                                            'value "default"))))
+                   'mentions-and-keywords (ement-alist
+                                           "override" nil
+                                           "room" (ement-alist
+                                                   'actions (vector "dont_notify")))
+                   'none (ement-alist
+                          "override" (ement-alist
+                                      'actions (vector "dont_notify")
+                                      'conditions (vector (ement-alist
+                                                           'kind "event_match"
+                                                           'key "room_id"
+                                                           'pattern (ement-room-id room))))
+                          "room" nil)))
+                 (kinds-and-rules (alist-get state available-states nil nil #'equal)))
+      (cl-loop with queue = (make-plz-queue :limit 1)
+               with total = (1- (length kinds-and-rules))
+               for count from 0
+               for message-fn = (if (equal count total)
+                                    (lambda (_data)
+                                      (message "Set notification rules for room: %s" (ement--format-room room)))
+                                  #'ignore)
+               for (kind . state) in kinds-and-rules
+               do (set-rule kind state queue message-fn)))))
+
 ;;;;; Public functions
 
 ;; These functions could reasonably be called by code in other packages.
 
+(cl-defun ement-put-state
+    (room type key data session
+          &key (then (lambda (response-data)
+                       (ement-debug "State data put on room" response-data data room session))))
+  "Put state event of TYPE with KEY and DATA on ROOM on SESSION.
+DATA should be an alist, which will become the JSON request
+body."
+  (declare (indent defun))
+  (pcase-let* ((endpoint (format "rooms/%s/state/%s/%s"
+                                 (url-hexify-string (ement-room-id room))
+                                 type key)))
+    (ement-api session endpoint :method 'put :data (json-encode data)
+      ;; TODO: Handle error codes.
+      :then then)))
+
 (defun ement-message (format-string &rest args)
   "Call `message' on FORMAT-STRING prefixed with \"Ement: \"."
   ;; TODO: Use this function everywhere we use `message'.
@@ -477,12 +723,14 @@ unseen user IDs to be input as well."
 	  selected-user))))
 
 (cl-defun ement-put-account-data
-    (session type data &key
+    (session type data &key room
              (then (lambda (received-data)
                      ;; Handle echoed-back account data event (the spec does not explain this,
                      ;; but see <https://github.com/matrix-org/matrix-react-sdk/blob/675b4271e9c6e33be354a93fcd7807253bd27fcd/src/settings/handlers/AccountSettingsHandler.ts#L150>).
                      ;; FIXME: Make session account-data a map instead of a list of events.
-                     (push received-data (ement-session-account-data session))
+                     (if room
+                         (push received-data (ement-room-account-data room))
+                       (push received-data (ement-session-account-data session)))
 
                      ;; NOTE: Commenting out this ement-debug form because a bug in Emacs
                      ;; causes this long string to be interpreted as the function's
@@ -492,10 +740,12 @@ unseen user IDs to be input as well."
                      ;;              (ement-user-id (ement-session-user session)) (json-encode data) received-data)
                      )))
   "Put account data of TYPE with DATA on SESSION.
-Also handle the echoed-back event."
+If ROOM, put it on that room's account data.  Also handle the
+echoed-back event."
   (declare (indent defun))
   (pcase-let* (((cl-struct ement-session (user (cl-struct ement-user (id user-id)))) session)
-               (endpoint (format "user/%s/account_data/%s" (url-hexify-string user-id) type)))
+               (room-part (if room (format "/rooms/%s" (ement-room-id room)) ""))
+               (endpoint (format "user/%s%s/account_data/%s" (url-hexify-string user-id) room-part type)))
     (ement-api session endpoint :method 'put :data (json-encode data)
       :then then)))
 
@@ -524,6 +774,10 @@ USER is an `ement-user' struct."
 ;; These functions aren't expected to be called by code in other packages (but if that
 ;; were necessary, they could be renamed accordingly).
 
+(defun ement--room-space-p (room)
+  "Return non-nil if ROOM is a space."
+  (equal "m.space" (ement-room-type room)))
+
 (cl-defun ement--prism-color (string &key (contrast-with (face-background 'default nil 'default)))
   "Return a computed color for STRING.
 The color is adjusted to have sufficient contrast with the color
@@ -707,9 +961,9 @@ period, anywhere in the body."
   "Return non-nil if EVENT mentions \"@room\"."
   (pcase-let (((cl-struct ement-event (content (map body))) event))
     (when body
-      (string-match-p (rx bow "@room" (or ":" (1+ blank))) body))))
+      (string-match-p (rx (or space bos) "@room" eow) body))))
 
-(cl-defun ement-complete-room (&key session predicate
+(cl-defun ement-complete-room (&key session (predicate #'identity)
                                     (prompt "Room: ") (suggest t))
   "Return a (room session) list selected from SESSION with completion.
 If SESSION is nil, select from rooms in all of `ement-sessions'.
@@ -726,6 +980,7 @@ suggested room."
                (name-to-room-session
                 (cl-loop for session in sessions
                          append (cl-loop for room in (ement-session-rooms session)
+                                         when (funcall predicate room)
                                          collect (cons (ement--format-room room 'topic)
                                                        (list room session)))))
                (names (mapcar #'car name-to-room-session))
@@ -735,7 +990,7 @@ suggested room."
                                  (when-let ((suggestion (ement--room-at-point)))
                                    (when (or (not predicate)
                                              (funcall predicate suggestion))
-                                     suggestion))))))
+                                     (ement--format-room suggestion 'topic)))))))
     (alist-get selected-name name-to-room-session nil nil #'string=)))
 
 (cl-defun ement-send-message (room session
@@ -776,6 +1031,8 @@ e.g. `ement-room-send-org-filter')."
                              :content content :data))))
 
 (defalias 'ement--button-buttonize
+  ;; FIXME: This doesn't set the mouse-face to highlight, and it doesn't use the
+  ;; default-button category.  Neither does `button-buttonize', of course, but why?
   (if (version< emacs-version "28.1")
       (lambda (string callback &optional data)
         "Make STRING into a button and return it.
@@ -805,6 +1062,7 @@ DATA is an unsent message event's data alist."
                (replying-to-sender-name (ement--user-displayname-in ement-room replying-to-sender))
                (quote-string (format "> <%s> %s\n\n" replying-to-sender-name replying-to-body))
                (reply-body (alist-get "body" data nil nil #'string=))
+               (reply-formatted-body (alist-get "formatted_body" data nil nil #'string=))
                (reply-body-with-quote (concat quote-string reply-body))
                (reply-formatted-body-with-quote
                 (format "<mx-reply>
@@ -821,7 +1079,7 @@ DATA is an unsent message event's data alist."
                         ;; might hope: there's `web-mode-html-entities' and `org-entities'.  See also
                         ;; <https://emacs.stackexchange.com/questions/8166/encode-non-html-characters-to-html-equivalent>.
                         (or replying-to-formatted-body replying-to-body)
-                        reply-body)))
+                        (or reply-formatted-body reply-body))))
     ;; NOTE: map-elt doesn't work with string keys, so we use `alist-get'.
     (setf (alist-get "body" data nil nil #'string=) reply-body-with-quote
           (alist-get "formatted_body" data nil nil #'string=) reply-formatted-body-with-quote
@@ -917,18 +1175,24 @@ m.replace metadata)."
   "Return ROOM formatted with name, alias, ID, and optionally TOPIC.
 Suitable for use in completion, etc."
   (if topic
-      (format "\"%s\" <%s> (<%s>): \"%s\""
+      (format "%s%s(<%s>)%s"
               (or (ement-room-display-name room)
                   (setf (ement-room-display-name room)
                         (ement--room-display-name room)))
-              (ement-room-canonical-alias room)
+              (if (ement-room-canonical-alias room)
+                  (format " <%s> " (ement-room-canonical-alias room))
+                " ")
               (ement-room-id room)
-              (ement-room-topic room))
-    (format "\"%s\" <%s> (<%s>)"
+              (if (ement-room-topic room)
+                  (format ": \"%s\"" (ement-room-topic room))
+                ""))
+    (format "%s%s(<%s>)"
             (or (ement-room-display-name room)
                 (setf (ement-room-display-name room)
                       (ement--room-display-name room)))
-            (ement-room-canonical-alias room)
+            (if (ement-room-canonical-alias room)
+                (format " <%s> " (ement-room-canonical-alias room))
+              " ")
             (ement-room-id room))))
 
 (defun ement--members-alist (room)
@@ -1003,16 +1267,16 @@ IMAGE should be one as created by, e.g. `create-image'."
 (declare-function eieio-oref "eieio-core")
 (defun ement--room-at-point ()
   "Return room at point.
-Works in major-modes `ement-room-mode', `ement-room-list-mode',
-and `ement-taxy-mode'."
+Works in major-modes `ement-room-mode',
+`ement-tabulated-room-list-mode', and `ement-room-list-mode'."
   (pcase major-mode
-    ('ement-room-mode (ement--format-room ement-room 'topic))
-    ('ement-room-list-mode (ement--format-room (tabulated-list-get-id) 'topic))
-    ('ement-taxy-mode
+    ('ement-room-mode ement-room)
+    ('ement-tabulated-room-list-mode (tabulated-list-get-id))
+    ('ement-room-list-mode
      (cl-typecase (oref (magit-current-section) value)
        (taxy-magit-section nil)
        (t (pcase (oref (magit-current-section) value)
-            (`[,room ,_session] (ement--format-room room 'topic))))))))
+            (`[,room ,_session] room)))))))
 
 (defun ement--room-direct-p (room session)
   "Return non-nil if ROOM on SESSION is a direct chat."
@@ -1083,6 +1347,11 @@ and `ement-taxy-mode'."
                (heroes joined)
                (format "%s, and %s others" (hero-names heroes)
                        (- joined (length heroes))))
+              (name-override
+               () (when-let ((event (alist-get "org.matrix.msc3015.m.room.name.override"
+                                               (ement-room-account-data room)
+                                               nil nil #'equal)))
+                    (map-nested-elt event '(content name))))
               (empty-room
                (heroes joined) (cl-etypecase (length heroes)
                                  ((satisfies zerop) "Empty room")
@@ -1090,7 +1359,8 @@ and `ement-taxy-mode'."
                                                        (hero-names heroes)))
                                  (t (format "Empty room (was %s)"
                                             (heroes-and-others heroes joined))))))
-    (or (latest-event "m.room.name" 'name)
+    (or (name-override)
+        (latest-event "m.room.name" 'name)
         (latest-event "m.room.canonical_alias" 'alias)
         (heroes-name)
         (member-events-name)
@@ -1139,7 +1409,8 @@ is not at the latest known message event."
                  (not (zerop highlight_count))))
         ;; NOTE: This is *WAY* too complicated, but it seems roughly equivalent to doesRoomHaveUnreadMessages() from
         ;; <https://github.com/matrix-org/matrix-react-sdk/blob/7fa01ffb068f014506041bce5f02df4f17305f02/src/Unread.ts#L52>.
-        (when timeline
+        (when (and (not ement-room-unread-only-counts-notifications)
+                   timeline)
           ;; A room should rarely, if ever, have a nil timeline, but in case it does
           ;; (which apparently can happen, given user reports), it should not be
           ;; considered unread.
